#ifndef FCTechnique_H
#define FCTechnique_H

#include <QObject>
#include <QTimer>
#include "fingercount.h"
#include "blobfingercount.h"
#include "blobradialstroke.h"

/**
  * Finger-Count Shortcuts states
  */
enum FCState
{
    CLOSED,
    EXPERT,
    NOVICE,
    CONTROL,
    CONFIRM
};

/**
\class FCTechnique
\brief Finger-Count Shortcuts technique helper class
nbFinger Left/Right and nbFinger Left/Right Expert are used to maintain the number of finger during timer.

nbFingerExpert save the max number of finger during the expert mode delay
nbFinger save the current number of finger during the expert mode delay in order to display the corresponding menu display in novice mode

nbFingerLeft is also used when the user releases the left hand in novice mode to let the user activate item with the right hand just after.
@author Gilles Bailly, Alexandre Viard
*/
class FCTechnique : public FingerCount
{
    Q_OBJECT
public:
    /**
     * Default Constructor
     */
    FCTechnique();
    //void resetAllPoints();
    /**
     * Provide pointers to the widget sending the commands selection and activation signal
     * \param left Finger-Count left hand widget
     * \param right Finger-Count right hand widget
     */
    void setBlobsFC(BlobFingerCount *left, BlobFingerCount *right);
    /**
     * Provide pointers to the widget sending the commands selection and activation signal
     * \param left Radial-Stroke left hand widget
     * \param right Radial-Stroke right hand widget
     */
    void setBlobsRS(BlobRadialStroke *left, BlobRadialStroke *right);

public slots:
    /**
     * Called when more fingers from the left hand are detected
     * \param nbF the number of fingers
     */
    void addLeftHandFinger(int);

    /**
    * Called when less fingers from the left hand are detected
    * \param nbF the number of fingers
    */
    void removeLeftHandFinger(int);

    /**
     * Called when the left hand is removed from the surface
     * \param nbF the number of fingers
     */
    void activateLeftHandFinger(int);

    /**
     * Triggered when the left hand move on the surface
     * \param p the new position 
     */
    void moveLeftHand(const QPoint& p);

    /**
     * Send the initial left hand position
     * \param p the inital position
     */
    void resetLeftHand(const QPoint& p);

    /**
    * Called when more fingers from the right hand are detected
    * \param nbF the number of fingers
    */
    void addRightHandFinger(int);

    /**
    * Called when more fingers from the right hand are detected
    * \param nbF the number of fingers
    */
    void removeRightHandFinger(int);

    /**
    * Called when the right hand is removed from the surface
    * \param nbF the number of fingers
    */
    void activateRightHandFinger(int);

    /**
    * Triggered when the right hand move on the surface
    * \param p the new position 
    */
    void moveRightHand(const QPoint& p);

    /**
     * Send the initial right hand position
     * \param  the inital position
     */
    void resetRightHand(const QPoint& p);

    /**
     * Close the QMenuBar and re-initialize the environment
     */
    void closeMenu();//pour le radialstroke patch

protected slots:
    /**
     *
     */
    void timerLeftHandFinished();
    
    /**
     *
     */
    void timerExpertFinished();

signals:
    /**
     * Provide the number of left hand fingers detected
     */
    void fingersLeftHand(int);
    
    /**
    * Provide the number of right hand fingers detected
    */
    void fingersRightHand(int);
    
    /**
     * Provide the new state
     */
    void stateChanged(QString stateStr, int stateI);
    
    /**
     * Send the new hands position
     */
    void handMoved(const QLine& left, const QLine& right);
    
    /**
     * Send the initial hand position
     */
    void handPositionInitialized(const QLine& left, const QLine& right);
    
    /**
     *The pan action is triggered 
     */
    void panActionStart();
    
    /**
    *The pan action is stopped
    */
    void panActionStop();
protected:
  
    /**
     * Change the current stateChanged
     * \param _state the new state
     */
    void setState(FCState _state) {
        state = _state;
    }
    
    /**
     * Change the current type of action selected, weither direct or relative
     */
    void setSecState(FCSecond _secState) {
        secState = _secState;
    }

    /**
     * Check the hand shifting to trigger the Relative or Direct Manipulation Finger-Count
     */
    void updateHandsPosition();
    
    /**
     *
     */
    void setFinishExpertMode();

    int nbLeftHandFinger, nbRightHandFinger;
    int nbLeftHandFingerExpert, nbRightHandFingerExpert;
    QTimer timerExpert, timerLeftHand;
    FCState state;
    FCSecond secState;
    int expertModeDelay, leftHandRemoveDelay;
    QLine leftPos, rightPos;
    int secondaryZone, continuousZone;
};

#endif // FCTechnique_H
