#ifndef FINGERCOUNT_H
#define FINGERCOUNT_H

#include <QMenuBar>
#include <QMap>
#include <QPair>

/**Relative Finger-Count Action type */
enum FCSecond
{
    DIRECT, /**< Standard Finger-Count Action*/
    UP,/**< Action positioned on top of a standard Finger-Finger-Count Action*/
    DOWN/**< Action positioned under of a standard Finger-Finger-Count Action*/
};

/**
\class FingerCount
\brief Finger-Count Shortcuts menu interface
This class povide methods to add QAction to the application QMenuBar
@author Gilles Bailly, Alexandre Viard
*/
class FingerCount:public QObject
{
    Q_OBJECT
public:
    /**Default Constructor*/
    FingerCount();

    /** Default Destructor*/
    virtual ~FingerCount();

    /** \return The associated menubar*/
    virtual QMenuBar* menuBar() {
        return m_menuBar;
    }

    /**
     *Set the associated menubar
     * \param menuBar the menu bar to extend
     */
    virtual void setMenuBar(QMenuBar *menuBar) {
        m_menuBar = menuBar;
        m_menuBar->setNativeMenuBar(false);
    }

    /**
     * Add a new menu at the end of the menubar
     * \param name the new menu name
     * \param fc the number of fingers needed (0 to disable the Finger-Count Shortcuts)
     * \return the new menu
     */
    virtual QMenu* addMenu(const QString & name="", int fc = 0);

    /**
    * Add a menu at the end of the menubar
    * \param menu the menu to add
    * \param fc the number of fingers needed (0 to disable the Finger-Count Shortcuts)
    * \return the new menu
    */
    virtual QMenu* addMenu(QMenu*, int fc =0);

    /**
    * insert a new menu before another in the menubar
    * \param before the action that will follow the new menu
    * \param name the new menu name
    * \param fc the number of fingers needed (0 to disable the Finger-Count Shortcuts)
    * \return the new menu
    */
    virtual QMenu* insertMenu(QAction* before , const QString & name="", int fc = 0);

    /**
    * insert a menu before another in the menubar
    * \param before the action that will follow the menu
    * \param menu the menu to add
    * \param fc the number of fingers needed (0 to disable the Finger-Count Shortcuts)
    * \return the new menu
    */
    virtual QMenu* insertMenu(QAction* before , QMenu*, int fc =0);

    /**
    * Add a new action at the end of the menu
    * \param menu the menu to modify
    * \param name the new action name
    * \param fc the number of fingers needed (0 to disable the Finger-Count Shortcuts)
    * \param discrete weither the action is discrete (true) or need direct manipulation (false)
    * \param secFav the Relative Finger-Count type 
    * \return the new action
    */
    virtual QAction* addAction(QMenu*, const QString& name = "", int fc =0, bool discrete = true,FCSecond secFav = DIRECT);
    
    /**
    * Add an action at the end of the current menu
    * \param act the action to add
    * \param fc the number of fingers needed (0 to disable the Finger-Count Shortcuts)
    * \param discrete weither the action is discrete (true) or need direct manipulation (false)
    * \param secFav the Relative Finger-Count type 
    * \return the new action
    */
    virtual QAction* addAction(QAction* act, int fc =0, bool discrete =true,FCSecond secFav = DIRECT); //if act already attached to an existing menu
    
    /**
    * Add an action at the end of the menu
    * \param menu the menu to modify
    * \param act the action to add
    * \param fc the number of fingers needed (0 to disable the Finger-Count Shortcuts)
    * \param discrete weither the action is discrete (true) or need direct manipulation (false)
    * \param secFav the Relative Finger-Count type 
    * \return the new action
    */
    virtual QAction* addAction(QMenu*, QAction*, int fc =0, bool discrete =true,FCSecond secFav = DIRECT);
    
    /**
    * Insert an action before another in the menu
    * \param before the action that will follow the new action
    * \param menu the menu to modify
    * \param name the new action name
    * \param fc the number of fingers needed (0 to disable the Finger-Count Shortcuts)
    * \param discrete weither the action is discrete (true) or need direct manipulation (false)
    * \param secFav the Relative Finger-Count type 
    * \return the new action
    */
    virtual QAction* insertAction(QAction* before , QMenu*, const QString& name = "", int fc =0, bool discrete = true,FCSecond secFav = DIRECT);
    
    /**
    * Insert an action before another in the menu
    * \param before the action that will follow the new action
    * \param menu the menu to modify
    * \param act the action to insert
    * \param fc the number of fingers needed (0 to disable the Finger-Count Shortcuts)
    * \param discrete weither the action is discrete (true) or need direct manipulation (false)
    * \param secFav the Relative Finger-Count type 
    * \return the new action
    */
    virtual QAction* insertAction(QAction* before , QMenu*, QAction*, int fc =0, bool discrete =true,FCSecond secFav = DIRECT);
    
    /**
     * Close the activated menu
     */
    virtual void reset();

signals:
    /**Emit the selected command name*/
    void commandSelected(const QString&);
    
    /**Emit the activated command name*/
    void commandActivated(const QString&);
    
    /**Emit the selected Direct Manipulation command name */
    void continuousCommandSelected(const QString&);
    
    /**Emit the activated Direct Manipulation command name*/
    void continuousCommandActivated(const QString&);

public slots:
    /**
     * Select and open the menu with the corresponding numbers
     * \param i the menu index
     */
    virtual QAction* selectMenu(int);
    
    /**
     * Select the action from the active menu using the current number of fingers
     * \param secondary the Relative Finger-Count action type
     * \param continuous weither the Direct Manipulation trigger were (true) or (false)
     */
    virtual QAction* selectItem(FCSecond secondary, bool continuous = false);
    
    /**
    * Select the action from the active menu
    * \param i the action index
    * \param secondary the Relative Finger-Count action type
    * \param continuous weither the Direct Manipulation trigger were (true) or (false)
    */
    virtual QAction* selectItem(int, FCSecond secondary =DIRECT, bool continuous = false);
    
    /**
    * Select the action from the specified menu
    * \param menu the menu index
    * \param item the action index
    * \param secondary the Relative Finger-Count action type
    * \param continuous weither the Direct Manipulation trigger were (true) or (false)
    */
    virtual QAction* selectItem(int, int, FCSecond secondary =DIRECT, bool continuous = false);
    
    /**
    * Activate the action from the active menu
    * \param i the action index
    * \param secondary the Relative Finger-Count action type
    */
    virtual QAction* activateItem(int, FCSecond secondary =DIRECT);
    
    /**
    * Activate the action from the specified menu
    * \param menu the menu index
    * \param item the action index
    * \param secondary the Relative Finger-Count action type
    */
    virtual QAction* activateItem(int menu, int item, FCSecond secondary =DIRECT);

protected:
    /**
     * Select the current finger-count action in the real menubar
     * \param continuous weither the Direct Manipulation trigger were (true) or (false)
     */
    virtual QAction* selectCurAction(bool continuous =false ); //return action from curIdAction and curIdMenu
    
    /**
     * Trigger the action from the menubar
     */
    virtual QAction* activateCurAction();
    
    QMenuBar * m_menuBar;
    QMap< int, QAction*> fingerAction; //protocole : key = fcMenu * 10 + fcFinger + 100 * {0 = DIRECT, 1 = UP, 2 = DOWN}
    QMap< int, QAction*> fingerContinuousAction; //protocole : key = fcMenu * 10 + fcFinger + 100 * {0 = DIRECT, 1 = UP, 2 = DOWN}
    int curIdMenu, curIdItem, curIdSec;
};

#endif // FINGERCOUNT_H
